/*****************************************************************
 * Project:         Software Reverse Engineering (SRE) Education
 *                  Malware, Adware, and Spyware
 *                  Alarm Clock Trojan (Malware) Application
 * Programmer:      Teodoro Cipresso, San Jose State University
 *                  tcipress@hotmail.com
 ******************************************************************
 */
package info.reversingproject.alarmclock;

import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.text.ParseException;
import java.util.Date;

/**
 * This class implements the command line interactive view for the Alarm Clock
 * application.
 */
public class AlarmClockView extends Thread {

	//
	private static final int MENU_DISPLAY_CLOCK_TIME = 1;
	private static final int MENU_DISPLAY_ALARM_TIME = 2;
	private static final int MENU_SET_ALARM_TIME = 3;
	private static final int MENU_QUIT_ALARM_CLOCK = 4;
	private static final int ALARM_REPEAT = 10;
	private static final int MENU_CHECK_INTERVAL = 500;
	//
	private AlarmClockModel alarmClockModel;
	private IAlarmClockViewListener alarmClockViewListener;
	private boolean menuVisible = false;
	private boolean alarmSounding = false;

	/**
	 * Display the menu when this instance is started on a new thread.
	 */

	public void run() {
		while (true) {
			try {
				Thread.yield();
				Thread.sleep(MENU_CHECK_INTERVAL);
			} catch (InterruptedException _exception) {
				_exception.printStackTrace();
			} // try
			if (!menuVisible && !alarmSounding) {
				displayMainMenu();
			} // if
		} // while
	} // runF

	/**
	 * Display the main menu to the user and wait for input.
	 */
	public void displayMainMenu() {
		menuVisible = true;
		System.out.println();
		System.out.println("+---------------------------------------------+");
		System.out.println("|            Alarm Clock V1.0                 |");
		System.out.println("+---------------------------------------------+");
		System.out.println("(1) Display the current date and time.");
		System.out.println("(2) Display the alarm date and time.");
		System.out.println("(3) Set the alarm date and time.");
		System.out.println("(4) Quit.");
		// get user menu selection
		System.out.println();
		System.out.print(">> Type an option number and press Enter: ");
		int selection = 0;
		try {
			selection = Integer.parseInt(getConsoleInput());
			// notify controller of user menu selection
			switch (selection) {
			case MENU_DISPLAY_CLOCK_TIME:
				alarmClockViewListener.displayClockTimeSelected();
				break;
			case MENU_DISPLAY_ALARM_TIME:
				alarmClockViewListener.displayAlarmTimeSelected();
				break;
			case MENU_SET_ALARM_TIME:
				alarmClockViewListener.setAlarmTimeSelected();
				break;
			case MENU_QUIT_ALARM_CLOCK:
				alarmClockViewListener.quitAlarmClockSelected();
				break;
			default:
				if (!alarmSounding) {
					displayUserMessageOnConsole(true,
							"An invalid menu option number" + " was entered.");
				}
			} // switch
		} catch (NumberFormatException _exception) {
			if (!alarmSounding) {
				displayUserMessageOnConsole(true,
						"A non-numeric value was specified.");
			}
		} // try
		menuVisible = false;
	} // displayMainMenu

	/**
	 * Display the current date and time to the console (user requested)
	 */
	public void displayClockTimeOnConsole() {
		// get the current time from the model and display it
		displayUserMessageOnConsole(
				false,
				"The current time is ("
						+ AlarmClockModel.getDateFormat().format(
								alarmClockModel.getClockTime()) + ").");
	} // displayClockTimeOnConsole

	/**
	 * Display the set alarm date and time to the console (user requested)
	 */
	public void displayAlarmTimeOnConsole(boolean isAlarmSet) {
		// display the alarm time if it's set
		if (isAlarmSet) {
			displayUserMessageOnConsole(
					false,
					"The alarm will sound at ("
							+ AlarmClockModel.getDateFormat().format(
									alarmClockModel.getAlarmTime()) + ").");
		} else {
			displayUserMessageOnConsole(true, "The alarm time is not set.");
		} // if
	} // displayAlarmTimeOnConsole

	/**
	 * Get the alarm date and time to set from the user. The prompt includes
	 * instructions on how to format the date and time appropriately.
	 */
	public Date getAlarmTimeInputFromConsole() {
		System.out.println();
		System.out
				.println(">> Specify the alarm date and time using the pattern ("
						+ "mm/dd/yy HH:MM:SS).");
		System.out.println(">> The current date and time is ("
				+ AlarmClockModel.getDateFormat().format(
						alarmClockModel.getClockTime()) + ").");
		System.out
				.print(">> Type the alarm date and time to set and press Enter ==> ");
		Date alarmTime = null;
		String consoleInput = getConsoleInput();
		try {
			alarmTime = AlarmClockModel.getDateFormat().parse(consoleInput);
		} catch (ParseException _exception) {
			displayUserMessageOnConsole(true, "The alarm time (" + consoleInput
					+ ") is not formatted correctly.  The alarm was not set.");
		} // try
		return alarmTime;
	} // getAlarmTimeInputFromConsole

	/**
	 * This method is used to display all informational or error messages to the
	 * console. Messages will have either and [ERROR] or [INFO] prefix.
	 */
	public void displayUserMessageOnConsole(boolean error, String reason) {
		String prefix = (error) ? "[ERROR] " : "[INFO] ";
		System.out.println();
		System.out.println(prefix + reason);
	} // displayUserMessageOnConsole

	/**
	 * This method implements the visual and auditory notifications for the
	 * alarm.
	 */
	public void doSoundAlarm() {
		alarmSounding = true;
		System.out.println();
		for (int repetition = 0; repetition < ALARM_REPEAT; repetition++) {
			displayUserMessageOnConsole(
					false,
					"<<ALARM>> THE TIME IS NOW ("
							+ AlarmClockModel.getDateFormat().format(
									alarmClockModel.getClockTime())
							+ ") <<ALARM>>");
			Toolkit.getDefaultToolkit().beep();
			try {
				Thread.sleep(1000);
			} catch (InterruptedException _exception) {
				_exception.printStackTrace();
			} // try
		} // for
		displayUserMessageOnConsole(false,
				"The alarm date and time has been cleared."
						+ " Press Enter to continue.");
		alarmSounding = false;
	} // doSoundAlarm

	/**
	 * Reads input typed by the user at the console. This method is used for
	 * reading menu selections and for pausing the console to allow the user to
	 * read messages.
	 */
	public String getConsoleInput() {
		String consoleInput = null;
		InputStreamReader inputReader = new InputStreamReader(System.in);
		BufferedReader bufferedReader = new BufferedReader(inputReader);
		try {
			consoleInput = bufferedReader.readLine();
		} catch (Exception _exception) {
			_exception.printStackTrace();
		} // if
		return consoleInput;
	} // getConsoleInput

	/**
	 * @param alarmClockModel
	 *            , singleton instance of the model
	 */
	public void setAlarmClockModel(AlarmClockModel alarmClockModel) {
		this.alarmClockModel = alarmClockModel;
	} // setAlarmClockModel

	/**
	 * @param alarmClockViewListener
	 *            , controller instance to notify of events on the view
	 */
	public void setAlarmClockViewListener(
			IAlarmClockViewListener alarmClockViewListener) {
		this.alarmClockViewListener = alarmClockViewListener;
	} // setAlarmClockViewListener

} // AlarmClockView

